/* ---------------------------------------------------------------------------
 * Copyright (C) 2014 ARM Limited. All rights reserved.
 *  
 * $Date:        23. January 2014
 * $Revision:    V1.00
 *  
 * Project:      I2C Definitions for LPC15xx
 * --------------------------------------------------------------------------*/

#include <stdint.h>

#ifndef __I2C_LPC15xx_H
#define __I2C_LPC15xx_H

typedef void (*I2C_CALLBK_T) (uint32_t err_code, uint32_t n);
typedef void* I2C_HANDLE_T;

typedef enum
{
  ERR_I2C_BASE = 0x00060000,
  ERR_I2C_NAK=ERR_I2C_BASE+1,           /*0x00060001*/ 
  ERR_I2C_BUFFER_OVERFLOW,              /*0x00060002*/ 
  ERR_I2C_BYTE_COUNT_ERR,               /*0x00060003*/ 
  ERR_I2C_LOSS_OF_ARBRITRATION,         /*0x00060004*/ 
  ERR_I2C_SLAVE_NOT_ADDRESSED,          /*0x00060005*/ 
  ERR_I2C_LOSS_OF_ARBRITRATION_NAK_BIT, /*0x00060006*/ 
  ERR_I2C_GENERAL_FAILURE,              /*0x00060007*/ 
  ERR_I2C_REGS_SET_TO_DEFAULT,          /*0x00060008*/ 
  ERR_I2C_TIMEOUT,                      /*0x00060009*/ 
  ERR_I2C_BUFFER_UNDERFLOW              /*0x0006000A*/ 
} ErrorCode_t;

typedef struct i2c_A {             //parameters passed to ROM function
  uint32_t num_bytes_send ;
  uint32_t num_bytes_rec ;
  uint8_t *buffer_ptr_send ;
  uint8_t *buffer_ptr_rec ;
  I2C_CALLBK_T func_pt; // callback function pointer
  uint8_t stop_flag;
  uint8_t dummy[3] ; // required for word alignment
} I2C_PARAM ;

typedef struct i2c_R { // RESULTs struct--results are here when returned
  uint32_t n_bytes_sent ;
  uint32_t n_bytes_recd ;
} I2C_RESULT ;

typedef enum I2C_mode {
  IDLE,
  MASTER_SEND,
  MASTER_RECEIVE,
  SLAVE_SEND,
  SLAVE_RECEIVE
} I2C_MODE_T ;

typedef void* I2C_HANDLE_T;


typedef struct I2CD_API { // index of all the i2c driver functions
  void (*i2c_isr_handler) (I2C_HANDLE_T* h_i2c) ; // ISR interrupt service request
  ErrorCode_t (*i2c_master_transmit_poll)(I2C_HANDLE_T* h_i2c, I2C_PARAM* ptp,  I2C_RESULT* ptr );
  ErrorCode_t (*i2c_master_receive_poll)(I2C_HANDLE_T* h_i2c, I2C_PARAM* ptp,   I2C_RESULT* ptr );
  ErrorCode_t (*i2c_master_tx_rx_poll)(I2C_HANDLE_T* h_i2c,I2C_PARAM* ptp,   I2C_RESULT* ptr ) ;
  ErrorCode_t (*i2c_master_transmit_intr)(I2C_HANDLE_T* h_i2c, I2C_PARAM* ptp,   I2C_RESULT* ptr ) ;
  ErrorCode_t (*i2c_master_receive_intr)(I2C_HANDLE_T* h_i2c, I2C_PARAM* ptp,   I2C_RESULT* ptr ) ;
  ErrorCode_t (*i2c_master_tx_rx_intr)(I2C_HANDLE_T* h_i2c, I2C_PARAM* ptp, I2C_RESULT*   ptr ) ;
  ErrorCode_t (*i2c_slave_receive_poll)(I2C_HANDLE_T* h_i2c, I2C_PARAM* ptp, I2C_RESULT*   ptr ) ;
  ErrorCode_t (*i2c_slave_transmit_poll)(I2C_HANDLE_T* h_i2c, I2C_PARAM* ptp,  I2C_RESULT* ptr ) ;
  ErrorCode_t (*i2c_slave_receive_intr)(I2C_HANDLE_T* h_i2c, I2C_PARAM* ptp, I2C_RESULT*   ptr ) ;
  ErrorCode_t (*i2c_slave_transmit_intr)(I2C_HANDLE_T* h_i2c, I2C_PARAM* ptp,   I2C_RESULT* ptr ) ;
  ErrorCode_t (*i2c_set_slave_addr)(I2C_HANDLE_T* h_i2c,   uint32_t slave_addr_0_3, uint32_t slave_mask_0_3);
  uint32_t (*i2c_get_mem_size)(void) ; //ramsize_in_bytes memory needed by I2C drivers
  I2C_HANDLE_T* (*i2c_setup)(uint32_t i2c_base_addr, uint32_t *start_of_ram ) ;
  ErrorCode_t (*i2c_set_bitrate)(I2C_HANDLE_T* h_i2c, uint32_t P_clk_in_hz,  uint32_t bitrate_in_bps) ;
  uint32_t (*i2c_get_firmware_version)() ;
  I2C_MODE_T (*i2c_get_status)(I2C_HANDLE_T* h_i2c ) ;
} I2CD_API_T ;

typedef struct _ROM {
   const unsigned pdev0;
   const unsigned pdev1;
   const unsigned pdev2;
   const unsigned pdev3;
   const unsigned pdev4;
   const I2CD_API_T *  pI2CD;
   const unsigned pdev6;
   const unsigned pdev7;
}  LPC_ROM_API_T;


#define rom_drivers_ptr (* (LPC_ROM_API_T **) 0x03000200)



#endif /* __I2C_LPC15xx_H */
