/* ----------------------------------------------------------------------
 * Copyright (C) 2014 ARM Limited. All rights reserved.
 *  
 * $Date:        31. 01 2014
 * $Revision:    V1.00
 *  
 * Project:      USB Driver definitions for NXP 15xx Series
 * -------------------------------------------------------------------- */

#ifndef USBD_LPC15XX_H_
#define USBD_LPC15XX_H_

#include <stdint.h>
#include <stdbool.h>
#include "LPC15xx.H"


// Endpoint command/status list structure
typedef struct {
  uint16_t  BufferAddrOffset;
  __IO uint16_t NBytes              : 10;
  uint16_t      Type                : 1;
  uint16_t      RateFeedback_Toogle : 1;
  uint16_t      ToogleReset         : 1;
  __IO uint16_t Stall               : 1;
  uint16_t      Disabled            : 1;
  __IO uint16_t Active              : 1;
} USB_CMD_STAT;

// Endpoint structure
typedef struct {
            uint8_t   *buffer;
            uint32_t   bufferIndex;
            uint32_t   dataSize;
  __packed  uint16_t   maxPacketSize;
  __packed  uint16_t   type;
} ENDPOINT;

// USB Device Command/Status register (DEVCMDSTAT, address 0x1C00 C000) bit description
#define USB_EN              (0x1 << 7)	/* Device Enable */
#define USB_SETUP_RCVD      (0x1 << 8)	/* SETUP token received */
#define USB_PLL_ON          (0x1 << 9)	/* PLL is always ON */
#define USB_LPM             (0x1 << 11)	/* LPM is supported */
#define USB_IntOnNAK_AO     (0x1 << 12)	/* Device Interrupt on NAK BULK OUT */
#define USB_IntOnNAK_AI     (0x1 << 13)	/* Device Interrupt on NAK BULK IN */
#define USB_IntOnNAK_CO     (0x1 << 14)	/* Device Interrupt on NAK CTRL OUT */
#define USB_IntOnNAK_CI     (0x1 << 15)	/* Device Interrupt on NAK CTRL IN */
#define USB_DCON            (0x1 << 16)	/* Device connect */
#define USB_DSUS            (0x1 << 17)	/* Device Suspend */
#define USB_LPM_SUS         (0x1 << 19)	/* LPM suspend */
#define USB_REMOTE_WAKE     (0x1 << 20)	/* LPM Remote Wakeup */
#define USB_DCON_C          (0x1 << 24)	/* Device connection change */
#define USB_DSUS_C          (0x1 << 25)	/* Device SUSPEND change */
#define USB_DRESET_C        (0x1 << 26)	/* Device RESET */
#define USB_VBUS_DBOUNCE    (0x1 << 28)	/* Device VBUS detect */

// USB interrupt enable register (INTEN, address 0x1C00 C024) bit description
#define EP0_INT             (0x1 << 0)
#define EP1_INT             (0x1 << 1)
#define EP2_INT             (0x1 << 2)
#define EP3_INT             (0x1 << 3)
#define EP4_INT             (0x1 << 4)
#define EP5_INT             (0x1 << 5)
#define EP6_INT             (0x1 << 6)
#define EP7_INT             (0x1 << 7)
#define EP8_INT             (0x1 << 8)
#define EP9_INT             (0x1 << 9)
#define FRAME_INT           (0x1 << 30)
#define DEV_STAT_INT        (0x80000000)


#endif /* USBD_LPC15XX_H_ */


/* --------------------------------- End Of File ------------------------------ */
